@@ -4,7 +4,7 @@ use warnings;
 package App::Spec;
 use 5.010;
 
-our $VERSION = '0.003_001'; # TRIAL VERSION
+our $VERSION = '0.003_002'; # TRIAL VERSION
 
 use App::Spec::Command;
 use App::Spec::Option;
@@ -30,13 +30,13 @@ my $DATA = do { local $/; <DATA> };
 my $default_spec;
 
 sub build {
-    my ($class, $spec) = @_;
-    for (@{ $spec->{options} || [] }, @{ $spec->{parameters} || [] }) {
+    my ($class, %spec) = @_;
+    for (@{ $spec{options} || [] }, @{ $spec{parameters} || [] }) {
         $_ = { spec => $_ } unless ref $_;
     }
-    $_ = App::Spec::Option->build($_) for @{ $spec->{options} || [] };
-    $_ = App::Spec::Parameter->build($_) for @{ $spec->{parameters} || [] };
-    my $self = $class->new($spec);
+    $_ = App::Spec::Option->build(%$_) for @{ $spec{options} || [] };
+    $_ = App::Spec::Parameter->build(%$_) for @{ $spec{parameters} || [] };
+    my $self = $class->new(%spec);
 }
 
 sub _read_default_spec {
@@ -95,15 +95,15 @@ sub read {
 
         for my $name (keys %{ $spec->{subcommands} || [] }) {
             my $cmd = $spec->{subcommands}->{ $name };
-            $commands->{ $name } = App::Spec::Command->build({
+            $commands->{ $name } = App::Spec::Command->build(
                 name => $name,
                 %$cmd,
-            });
+            );
         }
     }
 
     $spec->{subcommands} = $commands;
-    my $self = $class->build($spec);
+    my $self = $class->build(%$spec);
     return $self;
 }
 
@@ -155,20 +155,13 @@ sub usage {
     my ($self, %args) = @_;
     my $cmds = $args{commands};
     my %highlights = %{ $args{highlights} || {} };
-    my $color = $args{color};
+    my $colored = $args{colored} || sub { $_[1] };
     my $appname = $self->name;
 
-    if ($color) {
-        require Term::ANSIColor;
-    }
-
     my $abstract = $self->abstract // '';
     my $title = $self->title;
     my ($options, $parameters, $subcmds) = $self->_gather_options_parameters($cmds);
-    my $header = "$appname - $title";
-    if ($color) {
-        $header = Term::ANSIColor::colored([qw/ bold /], $header);
-    }
+    my $header = $colored->(['bold'], "$appname - $title");
     my $usage = <<"EOM";
 $header
 $abstract
@@ -176,10 +169,7 @@ $abstract
 EOM
 
     my $body = '';
-    my $usage_header = "Usage:";
-    if ($color) {
-        $usage_header = Term::ANSIColor::colored([qw/ bold /], $usage_header);
-    }
+    my $usage_header = $colored->([qw/ bold /], "Usage:");
     $usage .= "$usage_header $appname";
     $usage .= " @$cmds" if @$cmds;
     if (keys %$subcmds) {
@@ -187,12 +177,12 @@ EOM
         my @table;
         my $usage_string = "<subcommands>";
         my $header = "Subcommands:";
-        if ($color and $highlights{subcommands}) {
-            $usage_string = Term::ANSIColor::colored([qw/ bold red /], $usage_string);
-            $header = Term::ANSIColor::colored([qw/ bold red /], $header);
+        if ($highlights{subcommands}) {
+            $colored->([qw/ bold red /], $usage_string);
+            $colored->([qw/ bold red /], $header);
         }
-        elsif ($color) {
-            $header = Term::ANSIColor::colored([qw/ bold /], $header);
+        else {
+            $colored->([qw/ bold /], $header);
         }
         $usage .= " $usage_string";
         $body .= "$header\n";
@@ -230,18 +220,21 @@ EOM
         for my $param (@$parameters) {
             my $name = $param->name;
             my $highlight = $highlights{parameters}->{ $name };
-            push @highlights, ($color and $highlight) ? 1 : 0;
+            push @highlights, $highlight ? 1 : 0;
             my $summary = $param->summary;
             my $param_usage_header = $param->to_usage_header;
-            if ($color and $highlight) {
-                $param_usage_header = Term::ANSIColor::colored([qw/ bold red /], $param_usage_header);
+            if ($highlight) {
+                $colored->([qw/ bold red /], $param_usage_header);
             }
             $usage .= " " . $param_usage_header;
             my ($req, $multi) = (' ', '  ');
             if ($param->required) {
                 $req = "*";
             }
-            if ($param->multiple) {
+            if ($param->mapping) {
+                $multi = '{}';
+            }
+            elsif ($param->multiple) {
                 $multi = '[]';
             }
 
@@ -252,13 +245,10 @@ EOM
                 $maxlength = length $name;
             }
         }
-        my $parameters_string = "Parameters:";
-        if ($color) {
-            $parameters_string = Term::ANSIColor::colored([qw/ bold /], $parameters_string);
-        }
+        my $parameters_string = $colored->([qw/ bold /], "Parameters:");
         $body .= "$parameters_string\n";
         my @lines = $self->_output_table(\@table, [$maxlength]);
-        my $lines = $self->_colorize_lines(\@lines, \@highlights);
+        my $lines = $self->_colorize_lines(\@lines, \@highlights, $colored);
         $body .= $lines;
     }
 
@@ -270,7 +260,7 @@ EOM
         for my $opt (sort { $a->name cmp $b->name } @$options) {
             my $name = $opt->name;
             my $highlight = $highlights{options}->{ $name };
-            push @highlights, ($color and $highlight) ? 1 : 0;
+            push @highlights, $highlight ? 1 : 0;
             my $aliases = $opt->aliases;
             my $summary = $opt->summary;
             my @names = map {
@@ -284,7 +274,10 @@ EOM
             if ($opt->required) {
                 $req = "*";
             }
-            if ($opt->multiple) {
+            if ($opt->mapping) {
+                $multi = '{}';
+            }
+            elsif ($opt->multiple) {
                 $multi = '[]';
             }
 
@@ -292,13 +285,10 @@ EOM
 
             push @table, [$string, $req, $multi, $summary . $flags];
         }
-        my $options_string = "Options:";
-        if ($color) {
-            $options_string = Term::ANSIColor::colored([qw/ bold /], $options_string);
-        }
+        my $options_string = $colored->([qw/ bold /], "Options:");
         $body .= "\n$options_string\n";
         my @lines = $self->_output_table(\@table, [$maxlength]);
-        my $lines = $self->_colorize_lines(\@lines, \@highlights);
+        my $lines = $self->_colorize_lines(\@lines, \@highlights, $colored);
         $body .= $lines;
     }
 
@@ -314,6 +304,9 @@ sub _param_flags_string {
     if ($param->multiple) {
         push @flags, "multiple";
     }
+    if ($param->mapping) {
+        push @flags, "mapping";
+    }
     my $flags = @flags ? " (" . join("; ", @flags) . ")" : '';
     return $flags;
 }
@@ -331,12 +324,12 @@ sub generate_pod {
 }
 
 sub _colorize_lines {
-    my ($self, $lines, $highlights) = @_;
+    my ($self, $lines, $highlights, $colored) = @_;
     my $output = '';
     for my $i (0 .. $#$lines) {
         my $line = $lines->[ $i ];
         if ($highlights->[ $i ]) {
-            $line = Term::ANSIColor::colored([qw/ bold red /], $line);
+            $colored->([qw/ bold red /], $line);
         }
         $output .= $line;
     }
@@ -416,6 +409,10 @@ sub make_getopt {
             if ($opt->type eq 'flag') {
                 $spec .= '+';
             }
+            elsif ($opt->mapping) {
+                $result->{ $name } = {};
+                $spec .= '%';
+            }
             else {
                 $result->{ $name } = [];
                 $spec .= '@';
@@ -459,9 +456,7 @@ Your script:
     # this is equivalent to
     #my $run = App::Spec::Run->new(
     #    spec => $spec,
-    #    cmd => Your::App->new(
-    #        spec => $spec,
-    #    ),
+    #    cmd => Your::App->new,
     #);
     #$run->run;
 
@@ -471,10 +466,13 @@ Your App class:
     use base 'App::Spec::Run::Cmd';
 
     sub command1 {
-        my ($self) = @_;
-        my $options = $self->options;
-        my $param = $self->parameters;
+        my ($self, $run) = @_;
+        my $options = $run->options;
+        my $param = $run->parameters;
         # Do something
+        $run->out("Hello world!");
+        $run->err("oops");
+        # you can also use print directly
     }
 
 
@@ -496,7 +494,7 @@ Takes a file, hashref or glob and returns generated appspec hashref
 
 Builds objects out of the hashref
 
-    my $appspec = App::Spec->build($hashref);
+    my $appspec = App::Spec->build(%hash);
 
 =item runner
 
@@ -559,7 +557,7 @@ as perl itself.
 __DATA__
 options:
     -   name: help
-        description: Show command help
+        summary: Show command help
         type: flag
         aliases:
         - h
@@ -582,12 +580,12 @@ subcommands:
                         op: cmd_self_completion
                         options:
                             -   name: name
-                                description: name of the program (optional, override name in spec)
+                                summary: name of the program (optional, override name in spec)
                             -   name: zsh
-                                description: for zsh
+                                summary: for zsh
                                 type: flag
                             -   name: bash
-                                description: for bash
+                                summary: for bash
                                 type: flag
             pod:
                 summary: Pod documentation
