@@ -3,54 +3,54 @@ use strict;
 use warnings;
 package App::Spec::Argument;
 
-our $VERSION = '0.003_001'; # TRIAL VERSION
+our $VERSION = '0.003_002'; # TRIAL VERSION
 
 use Moo;
 
 has name => ( is => 'ro' );
 has type => ( is => 'ro' );
 has multiple => ( is => 'ro' );
+has mapping => ( is => 'ro' );
 has required => ( is => 'ro' );
 has unique => ( is => 'ro' );
 has summary => ( is => 'ro' );
 has description => ( is => 'ro' );
 has default => ( is => 'ro' );
-has filter => ( is => 'ro' );
 has completion => ( is => 'ro' );
 has enum => ( is => 'ro' );
 has values => ( is => 'ro' );
 
 sub common {
-    my ($class, $args) = @_;
+    my ($class, %args) = @_;
     my %dsl;
-    if (defined $args->{spec}) {
-        %dsl = $class->from_dsl(delete $args->{spec});
+    if (defined $args{spec}) {
+        %dsl = $class->from_dsl(delete $args{spec});
     }
-    my $description = $args->{description};
-    my $summary = $args->{summary};
-    $summary //= $description // '';
-    $description //= $summary;
-    my $type = $args->{type} // 'string';
+    my $description = $args{description};
+    my $summary = $args{summary};
+    $summary //= '';
+    $description //= '';
+    my $type = $args{type} // 'string';
     my %hash = (
-        name => $args->{name},
+        name => $args{name},
         summary => $summary,
         description => $description,
         type => $type,
-        multiple => $args->{multiple} ? 1 : 0,
-        required => $args->{required} ? 1 : 0,
-        unique => $args->{unique} ? 1 : 0,
-        default => $args->{default},
-        completion => $args->{completion},
-        enum => $args->{enum},
-        values => $args->{values},
-        filter => $args->{filter},
+        multiple => $args{multiple} ? 1 : 0,
+        mapping => $args{mapping} ? 1 : 0,
+        required => $args{required} ? 1 : 0,
+        unique => $args{unique} ? 1 : 0,
+        default => $args{default},
+        completion => $args{completion},
+        enum => $args{enum},
+        values => $args{values},
         %dsl,
     );
     not defined $hash{ $_ } and delete $hash{ $_ } for keys %hash;
     return %hash;
 }
 
-my $name_re = qr{\w+};
+my $name_re = qr{[\w-]+};
 
 sub from_dsl {
     my ($class, $dsl) = @_;
@@ -59,7 +59,8 @@ sub from_dsl {
     my $name;
     my $type = "flag";
     my $multiple = 0;
-    my$required = 0;
+    my $mapping = 0;
+    my $required = 0;
     $dsl =~ s/^\s+//;
 
     if ($dsl =~ s/^\+//) {
@@ -107,6 +108,10 @@ sub from_dsl {
     elsif ($type eq 'string' and $dsl =~ s/^\@//) {
         $multiple = 1;
     }
+    elsif ($type eq 'string' and $dsl =~ s/^\%//) {
+        $multiple = 1;
+        $mapping = 1;
+    }
 
     $dsl =~ s/^\s+//;
 
@@ -125,7 +130,6 @@ sub from_dsl {
     if ($dsl =~ s/^\s*--\s*(.*)//) {
         # TODO only summary should be supported
         $hash{summary} = $1;
-        $hash{description} = $1;
     }
 
     if (length $dsl) {
@@ -134,6 +138,7 @@ sub from_dsl {
 
     $hash{type} = $type;
     $hash{multiple} = $multiple;
+    $hash{mapping} = $mapping;
     $hash{required} = $required;
     return %hash;
 }
@@ -153,11 +158,11 @@ App::Spec::Argument - App::Spec objects representing command line options or par
 Options can be defined in a verbose way via key value pairs, but you can also
 use a shorter syntax.
 
+The idea comes from Ingy's L<http://www.schematype.org/>.
+
 The first item of the string is the name of the option using a syntax
 very similar to the one from L<Getopt::Long>.
 
-The idea comes from Ingy's L<http://www.schematype.org/>.
-
 Then you can optionally define a type, a default value and a summary.
 
 You can see a list of supported syntax in this example from C<t/data/12.dsl.yaml>:
@@ -168,26 +173,25 @@ START INLINE t/data/12.dsl.yaml
     ---
     # version with short dsl syntax
     name: myapp
-    appspec: { "version": 0.001 }
+    appspec: { version: 0.001 }
     class: App::Spec::Example::MyApp
     title: My Very Cool App
-    abstract: This app can do very cool things
     options:
-      - foo --Foo
-      - spec: verbose|v+ --be verbose
-      - spec: +req --Some required flag
-      - spec: number=i --integer option
+      - spec: foo                 --Foo
+      - spec: verbose|v+          --be verbose
+      - spec: +req                --Some required flag
+      - spec: number=i            --integer option
       - spec: number2|n= +integer --integer option
-      - date|d=s =today
-      - items=s@ --multi option
+      - spec: date|d=s =today
+      - spec: items=s@            --multi option
+      - spec: set=s%              --multiple key=value pairs
     
     ---
     # version with verbose syntax
     name: myapp
-    appspec: { "version": 0.001 }
+    appspec: { version: 0.001 }
     class: App::Spec::Example::MyApp
     title: My Very Cool App
-    abstract: This app can do very cool things
     options:
       - name: foo
         type: flag
@@ -216,6 +220,11 @@ START INLINE t/data/12.dsl.yaml
         type: string
         multiple: true
         summary: multi option
+      - name: set
+        type: string
+        multiple: true
+        mapping: true
+        summary: multiple key=value pairs
     
 
 
@@ -238,6 +247,11 @@ Builds a hash from the dsl string
 
     %dsl = $class->from_dsl("verbose|v+ --Be verbose");
 
+
+=item name, type, multiple, required, unique, summary, description, default, completion, enum, values, mapping
+
+Attributes which represent the ones from the spec.
+
 =back
 
 =cut
