@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package App::Spec::Command;
 
-our $VERSION = '0.003_001'; # TRIAL VERSION
+our $VERSION = '0.003_002'; # TRIAL VERSION
 
 use App::Spec::Option;
 use App::Spec::Parameter;
@@ -19,38 +19,38 @@ has subcommands => ( is => 'ro' );
 has subcommand_required => ( is => 'ro' );
 
 sub build {
-    my ($class, $args) = @_;
-    my $options = $args->{options} || [];
-    my $parameters = $args->{parameters} || [];
-    my $subcommands = $args->{subcommands} || {};
+    my ($class, %args) = @_;
+    my $options = $args{options} || [];
+    my $parameters = $args{parameters} || [];
+    my $subcommands = $args{subcommands} || {};
 
     my @options;
     my @parameters;
     my %subcommands;
 
     for my $opt (@$options) {
-        push @options, App::Spec::Option->build($opt);
+        push @options, App::Spec::Option->build(%$opt);
     }
     for my $p (@$parameters) {
-        push @parameters, App::Spec::Parameter->build($p);
+        push @parameters, App::Spec::Parameter->build(%$p);
     }
     for my $name (keys %$subcommands) {
         my $cmd = $subcommands->{ $name };
-        $subcommands{ $name } = App::Spec::Command->build({
+        $subcommands{ $name } = App::Spec::Command->build(
             name => $name,
             %$cmd
-        });
+        );
     }
 
     my $self = $class->new({
-        name => $args->{name},
-        summary => $args->{summary},
+        name => $args{name},
+        summary => $args{summary},
         options => \@options,
         parameters => \@parameters,
-        op => $args->{op},
+        op => $args{op},
         subcommands => \%subcommands,
-        description => $args->{description},
-        subcommand_required => $args->{subcommand_required},
+        description => $args{description},
+        subcommand_required => $args{subcommand_required},
     });
     return $self;
 }
