@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package App::Spec::Completion::Bash;
 
-our $VERSION = '0.003_001'; # TRIAL VERSION
+our $VERSION = '0.003_002'; # TRIAL VERSION
 
 use base 'App::Spec::Completion';
 
@@ -291,7 +291,7 @@ EOM
         my $type = $opt->type;
         next if $type eq "flag";
         my $enum = $opt->enum;
-        my $summary = $opt->description;
+        my $summary = $opt->summary;
         $summary =~ s/['`]/'"'"'/g;
         $summary =~ s/\$/\\\$/g;
         my $aliases = $opt->aliases;
