@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package App::Spec::Option;
 
-our $VERSION = '0.003_001'; # TRIAL VERSION
+our $VERSION = '0.003_002'; # TRIAL VERSION
 
 use base 'App::Spec::Argument';
 use Moo;
@@ -11,10 +11,10 @@ use Moo;
 has aliases => ( is => 'ro' );
 
 sub build {
-    my ($class, $args) = @_;
-    my %hash = $class->common($args);
+    my ($class, %args) = @_;
+    my %hash = $class->common(%args);
     my $self = $class->new({
-        aliases => $args->{aliases} || [],
+        aliases => $args{aliases} || [],
         %hash,
     });
     return $self;
@@ -38,11 +38,15 @@ This class inherits from L<App::Spec::Argument>
 
 =item build
 
-    my $option = App::Spec::Option->build({
+    my $option = App::Spec::Option->build(
         name => 'verbose',
         summary => 'lala',
         aliases => ['v'],
-    });
+    );
+
+=item aliases
+
+Attribute which represents the one from the spec.
 
 =back
 
