@@ -3,11 +3,12 @@ use strict;
 use warnings;
 package App::Spec::Run;
 use 5.010;
-our $VERSION = '0.003_001'; # TRIAL VERSION
+our $VERSION = '0.003_002'; # TRIAL VERSION
 
 use List::Util qw/ any /;
 use Data::Dumper;
 use App::Spec::Run::Validator;
+use App::Spec::Run::Response;
 use Getopt::Long qw/ :config pass_through bundling /;
 use Ref::Util qw/ is_arrayref /;
 use Moo;
@@ -20,10 +21,18 @@ has runmode => ( is => 'rw', default => 'normal' );
 has errors => ( is => 'rw' );
 has op => ( is => 'rw' );
 has cmd => ( is => 'rw' );
+has response => ( is => 'rw' );
+has validation_errors => ( is => 'rw' );
 
-sub run {
+sub process {
     my ($self) = @_;
 
+    my $res = $self->response;
+    unless ($res) {
+        $res = App::Spec::Run::Response->new;
+        $self->response($res);
+    }
+
     my $completion_parameter = $ENV{PERL5_APPSPECRUN_COMPLETION_PARAMETER};
     $self->check_help;
 
@@ -36,33 +45,84 @@ sub run {
         );
     }
 
-    my $opt = App::Spec::Run::Validator->new({
-        options => $self->options,
-        option_specs => \%option_specs,
-        parameters => $self->parameters,
-        param_specs => \%param_specs,
-    });
-    my %errs;
-    my ($ok) = $opt->process( \%errs, type => "parameters", app => $self );
-    $ok &&= $opt->process( \%errs, type => "options", app => $self );
-    $self->errors(\%errs) if not $ok;
-
-    if (not $ok and not $completion_parameter) {
-        $self->error_output;
-    }
-
-    my $op = $self->op;
-
-    if ($completion_parameter) {
-        $self->completion_output(
+    unless ($self->response->halted) {
+        my $opt = App::Spec::Run::Validator->new({
+            options => $self->options,
+            option_specs => \%option_specs,
+            parameters => $self->parameters,
             param_specs => \%param_specs,
-            completion_parameter => $completion_parameter,
-        );
-    }
-    else {
-        $self->cmd->$op($self);
+        });
+        my %errs;
+        my ($ok) = $opt->process( \%errs, type => "parameters", app => $self );
+        $ok &&= $opt->process( \%errs, type => "options", app => $self );
+        $self->errors(\%errs) if not $ok;
+        unless ($ok) {
+            $self->validation_errors(\%errs);
+            if (not $completion_parameter) {
+                $self->error_output;
+            }
+        }
     }
 
+    unless ($self->response->halted) {
+
+        my $op = $self->op;
+
+        if ($completion_parameter) {
+            $self->completion_output(
+                param_specs => \%param_specs,
+                completion_parameter => $completion_parameter,
+            );
+        }
+        else {
+            $self->run_op($op);
+        }
+    }
+
+}
+
+sub run {
+    my ($self) = @_;
+
+    $self->process;
+
+    $self->finish;
+
+}
+
+sub run_op {
+    my ($self, $op, $args) = @_;
+    $self->cmd->$op($self, $args);
+}
+
+sub out {
+    my ($self, $text) = @_;
+    my $res = $self->response;
+    $text .= "\n" unless $text =~ m/\n\z/;
+    $res->add_output($text);
+}
+
+sub err {
+    my ($self, $text) = @_;
+    my $res = $self->response;
+    $text .= "\n" unless $text =~ m/\n\z/;
+    $res->add_error($text);
+}
+
+sub halt {
+    my ($self, $exit) = @_;
+    $self->response->halted(1);
+    $self->response->exit($exit || 0);
+}
+
+sub finish {
+    my ($self) = @_;
+    my $res = $self->response;
+    $res->print_output;
+    $res->finished(1);
+    if (my $exit = $res->exit) {
+        exit $exit;
+    }
 }
 
 sub completion_output {
@@ -85,7 +145,7 @@ sub completion_output {
         runmode => "completion",
         parameter => $completion_parameter,
     };
-    my $result = $self->cmd->$op($self, $args);
+    my $result = $self->run_op($op, $args);
 
     my $string = '';
     my %seen;
@@ -108,7 +168,7 @@ sub completion_output {
         }
     }
 
-    print $string;
+    $self->out($string);
     return;
 }
 
@@ -130,30 +190,43 @@ sub error_output {
             push @error_output, $err;
         }
     }
-    $self->colorize and require Term::ANSIColor;
     my $help = $self->spec->usage(
         commands => $self->commands,
         highlights => $errs,
-        color => $self->colorize,
+        colored => $self->colorize_code('err'),
     );
-    say STDERR $help;
+    $self->err($help);
     for my $msg (@error_output) {
         $self->colorize
-            and $msg = Term::ANSIColor::colored([qw/ bold red /], $msg);
-        print STDERR "$msg\n";
+            and $msg = $self->colored('err', [qw/ error /], $msg);
+        $self->err("$msg\n");
     }
-    die "sorry =(\n";
+    $self->halt(1);
+}
+
+sub colorize_code {
+    my ($self, $out) = @_;
+    $self->colorize($out)
+        ? sub {
+            my $colored = $self->colored($out, $_[0], $_[1]);
+            unless (defined wantarray) {
+                $_[1] = $colored;
+            }
+            return $colored;
+        }
+        : sub { $_[1] },
 }
 
 sub colorize {
-    my ($self) = @_;
+    my ($self, $out) = @_;
+    $out ||= 'out';
     if (($ENV{PERL5_APPSPECRUN_COLOR} // '') eq 'always') {
         return 1;
     }
     if (($ENV{PERL5_APPSPECRUN_COLOR} // '') eq 'never') {
         return 0;
     }
-    if (-t STDOUT and -t STDERR) {
+    if ($out eq 'out' and -t STDOUT or $out eq 'err' and -t STDERR) {
         return 1;
     }
     return 0;
@@ -210,26 +283,29 @@ sub process_input {
         my $cmd = shift @ARGV;
         if (not defined $cmd) {
             if (not $op or $subcommand_required) {
-                warn $spec->usage(
+                $self->err($spec->usage(
                     commands => \@cmds,
-                    color => $self->colorize,
+                    colored => $self->colorize_code('err'),
                     highlights => {
                         subcommands => 1,
                     },
-                );
-                die $self->error("Missing subcommand(s)");
+                ));
+                $self->err( $self->error("Missing subcommand(s)") );
+                $self->halt(1);
             }
             last;
         }
         $cmd_spec = $commands->{ $cmd } or do {
-            warn $spec->usage(
+            $self->err($spec->usage(
                 commands => \@cmds,
-                color => $self->colorize,
+                colored => $self->colorize_code('err'),
                 highlights => {
                     subcommands => 1,
                 },
-            );
-            die $self->error("Unknown subcommand '$cmd'");
+            ));
+            $self->err( $self->error("Unknown subcommand '$cmd'") );
+            $self->halt(1);
+            last;
         };
         $subcommand_required = $cmd_spec->{subcommand_required} // 1;
         my $cmd_options = $cmd_spec->options;
@@ -244,32 +320,43 @@ sub process_input {
             param_specs => $param_specs,
         );
     }
-    my @names = sort keys %$commands;
-    unless ($op) {
-        if ($spec->has_subcommands) {
-            warn "Missing op for commands (@cmds)\n";
-            my $help = $spec->usage(
-                commands => \@cmds,
-                color => $self->colorize,
-            );
-            die $help;
-        }
-        else {
-            $op = "execute";
+
+    unless ($self->response->halted) {
+        unless ($op) {
+            if ($spec->has_subcommands) {
+                $self->err( "Missing op for commands (@cmds)\n" );
+                my $help = $spec->usage(
+                    commands => \@cmds,
+                    colored => $self->colorize_code('err'),
+                );
+                $self->err( $help );
+                $self->halt(1);
+            }
+            else {
+                $op = "execute";
+            }
         }
+        $self->commands(\@cmds);
+        $self->options(\%options);
+        $self->op($op);
+        return $op;
     }
-    $self->commands(\@cmds);
-    $self->options(\%options);
-    $self->op($op);
-    return $op;
+
+    return;
 }
 
 sub error {
     my ($self, $msg) = @_;
+    $msg = $self->colored('err', [qw/ error /], $msg) . "\n";
+}
+
+sub colored {
+    my ($self, $out, $colors, $msg) = @_;
+    $colors = [ map { $_ eq 'error' ? qw/ bold red / : $_ } @$colors ];
     require Term::ANSIColor;
-    $self->colorize
-        and $msg = Term::ANSIColor::colored([qw/ bold red /], $msg);
-    return "$msg\n";
+    $self->colorize($out)
+        and $msg = Term::ANSIColor::colored($colors, $msg);
+    return $msg;
 }
 
 sub check_help {
@@ -303,9 +390,9 @@ sub cmd_help {
     shift @$cmds;
     my $help = $spec->usage(
         commands => $cmds,
-        color => $run->colorize,
+        colored => $run->colorize_code,
     );
-    say $help;
+    $run->out($help);
 }
 
 sub cmd_self_completion {
@@ -325,7 +412,7 @@ sub cmd_self_completion {
     my $completion = $spec->generate_completion(
         shell => $shell,
     );
-    say $completion;
+    $run->out($completion);
 }
 
 sub cmd_self_pod {
@@ -333,7 +420,7 @@ sub cmd_self_pod {
     my $spec = $run->spec;
     my $pod = $spec->generate_pod(
     );
-    say $pod;
+    $run->out($pod);
 }
 
 1;
