@@ -3,14 +3,24 @@ use warnings;
 use strict;
 use 5.010;
 
+use Ref::Util qw/ is_arrayref is_hashref /;
+
 use base 'App::Spec::Run::Cmd';
 
 sub _dump_hash {
     my ($self, $hash) = @_;
     my @strings;
     for my $key (sort keys %$hash) {
-    next unless defined $hash->{ $key };
-        push @strings, "$key=$hash->{ $key }";
+        next unless defined $hash->{ $key };
+        my $value = $hash->{ $key };
+        if (is_hashref($value)) {
+            for my $key2 (sort keys %$value) {
+                push @strings, "$key=($key2=$value->{ $key2 })";
+            }
+        }
+        else {
+            push @strings, "$key=$value";
+        }
     }
     return join ",", @strings;
 }
@@ -20,9 +30,9 @@ sub cook {
     my $param = $run->parameters;
     my $opt = $run->options;
     if ($ENV{PERL5_APPSPECRUN_TEST}) {
-        say "Subcommands: cook";
-        say "Options: " . $self->_dump_hash($opt);
-        say "Parameters: " .  $self->_dump_hash($param);
+        $run->out("Subcommands: cook");
+        $run->out("Options: " . $self->_dump_hash($opt));
+        $run->out("Parameters: " .  $self->_dump_hash($param));
         return;
     }
 
@@ -34,8 +44,8 @@ sub cook {
     push @with, $with if $with;
     push @with, "sugar" if $opt->{sugar};
 
-    say "Starting to cook $param->{drink}"
-    . (@with ? " with ". (join " and ", @with) : '');
+    $run->out("Starting to cook $param->{drink}"
+    . (@with ? " with ". (join " and ", @with) : ''));
 }
 
 my %countries = (
@@ -175,13 +185,24 @@ sub convert {
         my $sub = $conversions{ $type }->{ $key };
         my $result = $sub->($value);
         my $label = $units{ $type }->{ $target }->{label};
-        say "$result$label";
+        $run->out("$result$label");
     }
 }
 
+sub config {
+    my ($self, $run, $args) = @_;
+    my $opt = $run->options;
+    my $param = $run->parameters;
+    if ($ENV{PERL5_APPSPECRUN_TEST}) {
+        $run->out("Options: " . $self->_dump_hash($opt));
+        return;
+    }
+    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$opt], ['opt']);
+}
 
 sub convert_complete {
     my ($self, $run, $args) = @_;
+    my $errors = $run->validation_errors;
     my $runmode = $args->{runmode};
     return if ($runmode ne "completion" and $runmode ne "validation");
     my $comp_param = $args->{parameter};
@@ -190,19 +211,35 @@ sub convert_complete {
     if ($comp_param eq 'type') {
         return [sort keys %units];
     }
-    elsif ($comp_param eq 'source') {
+    if (delete $errors->{parameters}->{type}) {
+        $run->err("Invalid type\n");
+        return;
+    }
+    if ($comp_param eq 'source') {
         my $type = $param->{type};
         my $units = $units{ $type };
         return [map {
             +{ name => $_, description => $units->{ $_ }->{label} }
         } keys %$units];
     }
-    elsif ($comp_param eq 'target') {
+    if (delete $errors->{parameters}->{source}) {
+        $run->err("Invalid source\n");
+        return;
+    }
+    delete $errors->{parameters}->{target};
+    if (delete $errors->{parameters}->{value}) {
+        $run->err("Invalid value\n");
+        return;
+    }
+    if ($comp_param eq 'target') {
         my $type = $param->{type};
         my $source = $param->{source};
         my $value = $param->{value};
         my $units = $units{ $type };
         my @result;
+        if ($runmode eq "validation") {
+            return [sort keys %$units];
+        }
         for my $unit (sort keys %$units) {
             next if $unit eq $source;
             my $label = $units->{ $unit }->{label};
